searchData={"items":[{"type":"module","title":"Francis","doc":"Module responsible for starting the Francis server and to wrap the Plug functionality\n\nThis module performs multiple tasks:\n  * Uses the Application module to start the Francis server\n  * Defines the Francis.Router which uses Francis.Plug.Router, :match and :dispatch\n  * Defines the macros get, post, put, delete, patch and ws to define routes for each operation\n  * Setups Plug.Static with the given options\n  * Sets up Plug.Parsers with the default configuration of:\n    * ```elixir\n      plug(Plug.Parsers,\n        parsers: [:urlencoded, :multipart, :json],\n        json_decoder: Jason\n      )\n      ```\n  * Defines a default error handler that returns a 500 status code and a generic error message. You can override this by passing the function name on `:error_handler` option to the `use Francis` macro which will override the default error handler.\n\nYou can also set the following options:\n  * :bandit_opts - Options to be passed to Bandit\n  * :static - Configure Plug.Static to serve static files\n  * :parser - Overrides the default configuration for Plug.Parsers\n  * :error_handler - Defines a custom error handler for the server\n  * :log_level - Sets the log level for Plug.Logger (default is `:info`)","ref":"Francis.html"},{"type":"macro","title":"Francis.delete/2","doc":"Defines a DELETE route","ref":"Francis.html#delete/2"},{"type":"macro","title":"Examples - Francis.delete/2","doc":"```elixir\ndefmodule Example.Router do\n  use Francis\n\n  delete \"/hello\", fn conn ->\n    \"Hello World!\"\n  end\nend\n```","ref":"Francis.html#delete/2-examples"},{"type":"macro","title":"Francis.get/2","doc":"Defines a GET route","ref":"Francis.html#get/2"},{"type":"macro","title":"Examples - Francis.get/2","doc":"```elixir\ndefmodule Example.Router do\n  use Francis\n\n  get \"/hello\", fn conn ->\n    \"Hello World!\"\n  end\nend\n```","ref":"Francis.html#get/2-examples"},{"type":"function","title":"Francis.get_configuration/3","doc":"Retrieves the configuration for a given key, checking both the macro options and the application environment.","ref":"Francis.html#get_configuration/3"},{"type":"macro","title":"Francis.patch/2","doc":"Defines a PATCH route","ref":"Francis.html#patch/2"},{"type":"macro","title":"Examples - Francis.patch/2","doc":"```elixir\ndefmodule Example.Router do\n  use Francis\n\n  patch \"/hello\", fn conn ->\n    \"Hello World!\"\n  end\nend\n```","ref":"Francis.html#patch/2-examples"},{"type":"macro","title":"Francis.post/2","doc":"Defines a POST route","ref":"Francis.html#post/2"},{"type":"macro","title":"Examples - Francis.post/2","doc":"```elixir\ndefmodule Example.Router do\n  use Francis\n\n  post \"/hello\", fn conn ->\n    \"Hello World!\"\n  end\nend\n```","ref":"Francis.html#post/2-examples"},{"type":"macro","title":"Francis.put/2","doc":"Defines a PUT route","ref":"Francis.html#put/2"},{"type":"macro","title":"Examples - Francis.put/2","doc":"```elixir\ndefmodule Example.Router do\n  use Francis\n\n  put \"/hello\", fn conn ->\n    \"Hello World!\"\n  end\nend\n```","ref":"Francis.html#put/2-examples"},{"type":"macro","title":"Francis.unmatched/1","doc":"Defines an action for umatched routes and returns 404","ref":"Francis.html#unmatched/1"},{"type":"macro","title":"Francis.ws/3","doc":"Defines a WebSocket route that sends text type responses.\n\nThe handler function receives the message and the socket state, and it can return a binary or a map.\nThe state includes:\n- `:transport` - The transport process that can be used to send messages back to the client using `send/2`\n- `:id` - A unique identifier for the WebSocket connection that can be used to track the connection\n- `:path` - The path of the WebSocket connection to identify the route that triggered the connection","ref":"Francis.html#ws/3"},{"type":"macro","title":"Examples - Francis.ws/3","doc":"```elixir\ndefmodule Example.Router do\n  use Francis\n\n  ws \"/hello\", fn _, socket ->\n    \"Hello World!\"\n  end\nend\n```","ref":"Francis.html#ws/3-examples"},{"type":"module","title":"Francis.Plug.Router","doc":"Module that imports required macros from Francis and Plug.Router","ref":"Francis.Plug.Router.html"},{"type":"module","title":"Francis.ResponseHandlers","doc":"A module providing functions to handle HTTP responses in a Plug application.","ref":"Francis.ResponseHandlers.html"},{"type":"function","title":"Francis.ResponseHandlers.html/2","doc":"Sends an HTML response with the given status code and HTML content.\n\n**Warning:** The following function does **not** escape HTML content.\nPassing user-generated or untrusted input may result in [Cross-Site Scripting (XSS)](https://owasp.org/www-community/attacks/xss/) vulnerabilities.\nOnly use this function with trusted, static HTML content. Look into [phoenix_html](https://github.com/phoenixframework/phoenix_html/)","ref":"Francis.ResponseHandlers.html#html/2"},{"type":"function","title":"Examples - Francis.ResponseHandlers.html/2","doc":"```elixir\nhtml(conn, 200, \" Hello World! \")\n```","ref":"Francis.ResponseHandlers.html#html/2-examples"},{"type":"function","title":"Francis.ResponseHandlers.html/3","doc":"Sends an HTML response with the given status code and HTML content.\n\n**Warning:** The following function does **not** escape HTML content.\nPassing user-generated or untrusted input may result in [Cross-Site Scripting (XSS)](https://owasp.org/www-community/attacks/xss/) vulnerabilities.\nOnly use this function with trusted, static HTML content. Look into [phoenix_html](https://github.com/phoenixframework/phoenix_html/)","ref":"Francis.ResponseHandlers.html#html/3"},{"type":"function","title":"Examples - Francis.ResponseHandlers.html/3","doc":"```elixir\nhtml(conn, 200, \" Hello World! \")\n```","ref":"Francis.ResponseHandlers.html#html/3-examples"},{"type":"function","title":"Francis.ResponseHandlers.json/2","doc":"Sends a JSON response with a 200 status code and the given data.","ref":"Francis.ResponseHandlers.html#json/2"},{"type":"function","title":"Examples - Francis.ResponseHandlers.json/2","doc":"```elixir\njson(conn, %{message: \"Success\"})\n```","ref":"Francis.ResponseHandlers.html#json/2-examples"},{"type":"function","title":"Francis.ResponseHandlers.json/3","doc":"Sends a JSON response with the given status code and data.","ref":"Francis.ResponseHandlers.html#json/3"},{"type":"function","title":"Examples - Francis.ResponseHandlers.json/3","doc":"```elixir\njson(conn, 201, %{message: \"Success\"})\n```","ref":"Francis.ResponseHandlers.html#json/3-examples"},{"type":"function","title":"Francis.ResponseHandlers.redirect/2","doc":"Redirects the connection to the specified path with a 302 status code.","ref":"Francis.ResponseHandlers.html#redirect/2"},{"type":"function","title":"Examples - Francis.ResponseHandlers.redirect/2","doc":"```elixir\nredirect(conn, \"/new_path\")\n```","ref":"Francis.ResponseHandlers.html#redirect/2-examples"},{"type":"function","title":"Francis.ResponseHandlers.redirect/3","doc":"Redirects the connection to the specified path with a custom status code.","ref":"Francis.ResponseHandlers.html#redirect/3"},{"type":"function","title":"Examples - Francis.ResponseHandlers.redirect/3","doc":"```elixir\nredirect(conn, 301, \"/new_path\")\n```","ref":"Francis.ResponseHandlers.html#redirect/3-examples"},{"type":"function","title":"Francis.ResponseHandlers.text/2","doc":"Sends a text response with a 200 status code and the given text.","ref":"Francis.ResponseHandlers.html#text/2"},{"type":"function","title":"Examples - Francis.ResponseHandlers.text/2","doc":"```elixir\ntext(conn, \"Hello World!\")\n```","ref":"Francis.ResponseHandlers.html#text/2-examples"},{"type":"function","title":"Francis.ResponseHandlers.text/3","doc":"Sends a text response with the given status code and text.","ref":"Francis.ResponseHandlers.html#text/3"},{"type":"function","title":"Examples - Francis.ResponseHandlers.text/3","doc":"```elixir\ntext(conn, 200, \"Hello World!\")\n```","ref":"Francis.ResponseHandlers.html#text/3-examples"},{"type":"module","title":"Francis.Watcher","doc":"Watcher mode for development that checks modified time of your files and recompiles them","ref":"Francis.Watcher.html"},{"type":"function","title":"Francis.Watcher.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Francis.Watcher.html#child_spec/1"},{"type":"function","title":"Francis.Watcher.init/1","doc":"Callback implementation for  GenServer.init/1 .","ref":"Francis.Watcher.html#init/1"},{"type":"function","title":"Francis.Watcher.start_link/1","doc":"","ref":"Francis.Watcher.html#start_link/1"},{"type":"task","title":"mix francis.new","doc":"Generates a new Francis project.\nThis task creates a new directory with the specified application name and a basic project structure.","ref":"Mix.Tasks.Francis.New.html"},{"type":"task","title":"Usage - mix francis.new","doc":"mix francis.new   [--sup [ ]]","ref":"Mix.Tasks.Francis.New.html#module-usage"},{"type":"task","title":"Examples - mix francis.new","doc":"mix francis.new my_app\n    mix francis.new my_app --sup\n    mix francis.new my_app --sup MyApp","ref":"Mix.Tasks.Francis.New.html#module-examples"},{"type":"function","title":"Mix.Tasks.Francis.New.main/1","doc":"","ref":"Mix.Tasks.Francis.New.html#main/1"},{"type":"function","title":"Mix.Tasks.Francis.New.run/1","doc":"Callback implementation for  Mix.Task.run/1 .","ref":"Mix.Tasks.Francis.New.html#run/1"},{"type":"function","title":"Mix.Tasks.Francis.New.usage/0","doc":"","ref":"Mix.Tasks.Francis.New.html#usage/0"},{"type":"task","title":"mix francis.release","doc":"Generates Docker deployment files for Francis projects.\n\nThis task creates a `Dockerfile` and `.dockerignore` in the current working directory,\nusing the project name and configuration. It is intended to simplify containerization\nand deployment of Francis-based applications.","ref":"Mix.Tasks.Francis.Release.html"},{"type":"task","title":"Usage - mix francis.release","doc":"mix francis.release [OPTIONS]","ref":"Mix.Tasks.Francis.Release.html#module-usage"},{"type":"task","title":"Command line options - mix francis.release","doc":"* `-p`, `--port  ` - Port to expose in the Docker container (default: 4000).\n  * `--elixir-version  ` - Elixir version to use in the Docker image (default: 1.18.4).\n  * `--otp-version  ` - Erlang/OTP version to use in the Docker image (default: 27.3.4).\n\nExample:\n\n    mix francis.release --port 8080 --elixir-version 1.16.2 --otp-version 26.2.1\n\nThis will generate a Dockerfile exposing port 8080 and using the specified\nElixir and OTP versions. If you use some combination of versions that are not\ncompatible it will fail when building the docker image.\n\nAll files are generated in the current working directory. Unknown options are ignored.","ref":"Mix.Tasks.Francis.Release.html#module-command-line-options"},{"type":"callback","title":"Mix.Tasks.Francis.Release.generate_files/1","doc":"","ref":"Mix.Tasks.Francis.Release.html#c:generate_files/1"},{"type":"task","title":"mix francis.server","doc":"Starts the application by configuring all endpoints servers to run.\n\nNote: to start the endpoint without using this mix task you must set\n`server: true` in your `Phoenix.Endpoint` configuration.","ref":"Mix.Tasks.Francis.Server.html"},{"type":"task","title":"Command line options - mix francis.server","doc":"Task accepts the same command-line options as `mix run`.\n\nThe `--no-halt` flag is automatically added.","ref":"Mix.Tasks.Francis.Server.html#module-command-line-options"},{"type":"extras","title":"Francis","doc":"# Francis\n\n[![Hex version badge](https://img.shields.io/hexpm/v/francis.svg)](https://hex.pm/packages/francis)\n[![License badge](https://img.shields.io/hexpm/l/repo_example.svg)](https://github.com/francis-build/francis/blob/master/LICENSE.md)\n[![Elixir CI](https://github.com/francis-build/francis/actions/workflows/elixir.yaml/badge.svg)](https://github.com/francis-build/francis/actions/workflows/elixir.yaml)\n\nSimple boilerplate killer using Plug and Bandit inspired by [Sinatra](https://sinatrarb.com) for Ruby.\n\nFocused on reducing time to build as it offers automatic request parsing, automatic response parsing, easy DSL to build quickly new endpoints and websocket listeners.","ref":"readme.html"},{"type":"extras","title":"Installation - Francis","doc":"If [available in Hex](https://hex.pm/docs/publish), the package can be installed by adding `francis` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:francis, \"~> 0.1.0\"}\n  ]\nend\n```\n\nYou can also use the Francis generator to create all the initial project files. You need to install the francis tasks first.\n\n```bash\nmix archive.install hex francis\n```\n\nThen you can create a new project with:\n\n```bash\nmix francis.new my_app\n```\n\nUse `mix help francis.new` to see all the available options.","ref":"readme.html#installation"},{"type":"extras","title":"Usage - Francis","doc":"To start the server up you can run `mix francis.server` or if you need a iex console you can run with `iex -S mix francis.server`.","ref":"readme.html#usage"},{"type":"extras","title":"Deployment - Francis","doc":"To create the Dockerfile that can be used for deployment you can run:\n\n```bash\nmix francis.release\n```","ref":"readme.html#deployment"},{"type":"extras","title":"Configuration - Francis","doc":"You can configure Francis in your `config/config.exs` file. The following options are available:\n\n- `dev` - If set to `true`, it will enable the development mode which will automatically reload the server when you change your code. Defaults to `false`.\n- `bandit_opts` - Options to be passed to Bandit\n- `static` - Configure Plug.Static to serve static files\n- `parser` - Overrides the default configuration for Plug.Parsers\n- `error_handler` - Defines a custom error handler for the server\n- `log_level` - Sets the log level for Plug.Logger (default is `:info`)\n\n```elixir\nimport Config\n\nconfig :francis,\n  dev: false,\n  bandit_opts: [port: 4000],\n  static: [from: \"priv/static\", at: \"/\"],\n  parser: [parsers: [:json, :urlencoded], pass: [\"*/*\"]],\n  error_handler: &Example.error/2,\n  log_level: :info\n```\n\nYou can also set the values in `use` macro:\n\n```elixir\ndefmodule Example do\n  use Francis,\n    dev: true,\n    bandit_opts: [port: 4000],\n    static: [from: \"priv/static\", at: \"/\"],\n    parser: [parsers: [:json, :urlencoded], pass: [\"*/* \"]],\n    error_handler: &Example.error/2,\n    log_level: :info\nend\n```","ref":"readme.html#configuration"},{"type":"extras","title":"Error Handling - Francis","doc":"By default, Francis will return a 500 error with the message \"Internal Server Error\" if you return a tuple `{:error, any()}` or an exception is raised during the request handling.","ref":"readme.html#error-handling"},{"type":"extras","title":"Unmatched Routes - Francis","doc":"If a request does not match any defined route, you can use the `unmatched/1` macro to define a custom response:\n\n```elixir\nunmatched(fn _conn -> \"not found\" end)\n```","ref":"readme.html#unmatched-routes"},{"type":"extras","title":"Custom Error Responses - Francis","doc":"For more advanced error handling, you can setup a custom error handler by providing the function that will handle the errors of your application:\n\n```elixir\ndefmodule Example do\n  use Francis, error_handler: &__MODULE__.error/2\n\n  get(\"/\", fn _ -> {:error, :potato} end)\n\n  def error(conn,{:error, :failed}) do\n    # Return a custom response\n    Plug.Conn.send_resp(conn, 502, \"Custom error response\")\n  end\nend\n```\n\nIf you do not handle errors explicitly, Francis will catch them and return a 500 response.","ref":"readme.html#custom-error-responses"},{"type":"extras","title":"Example of a router - Francis","doc":"```elixir\ndefmodule Example do\n  use Francis\n\n  get(\"/\", fn _ -> \" world \" end)\nend\n```\n\nIf you do not handle errors explicitly, Francis will catch them and return a 500 response.","ref":"readme.html#example-of-a-router"},{"type":"extras","title":"Example of a router - Francis","doc":"```elixir\ndefmodule Example do\n  use Francis\n\n  get(\"/\", fn _ -> \" world \" end)\n  get(\"/:name\", fn %{params: %{\"name\" => name}} -> \"hello #{name}\" end)\n  post(\"/\", fn conn -> conn.body_params end)\n\n  ws(\"ws\", fn \"ping\" -> \"pong\" end)\n\n  unmatched(fn _ -> \"not found\" end)\nend\n```\n\nAnd in your `mix.exs` file add that this module should be the one used for\nstartup:\n\n```elixir\ndef application do\n  [\n    extra_applications: [:logger],\n    mod: {Example, []}\n  ]\nend\n```\n\nThis will ensure that Mix knows what module should be the entrypoint.","ref":"readme.html#example-of-a-router-1"},{"type":"extras","title":"Example of a router with Static serving - Francis","doc":"With the `static` option, you are able to setup the options for `Plug.Static` to serve static assets easily.\n\n```elixir\ndefmodule Example do\n  use Francis, static: [from: \"priv/static\", at: \"/\"]\nend\n```","ref":"readme.html#example-of-a-router-with-static-serving"},{"type":"extras","title":"Example of a router with Plugs - Francis","doc":"With the `plugs` option you are able to apply a list of plugs that happen\nbetween before dispatching the request.\n\nIn the following example we're adding the `Plug.BasicAuth` plug to setup basic\nauthentication on all routes\n\n```elixir\ndefmodule Example do\n  import Plug.BasicAuth\n\n  use Francis\n\n  plug(:basic_auth, username: \"test\", password: \"test\")\n\n  get(\"/\", fn _ -> \" world \" end)\n  get(\"/:name\", fn %{params: %{\"name\" => name}} -> \"hello #{name}\" end)\n\n  ws(\"ws\", fn \"ping\", _socket -> \"pong\" end)\n\n  unmatched(fn _ -> \"not found\" end)\nend\n```","ref":"readme.html#example-of-a-router-with-plugs"},{"type":"extras","title":"Example of multiple routers - Francis","doc":"You can also define multiple routers in your application by using the `forward/2` function provided by [Plug](https://hexdocs.pm/plug/Plug.Router.html#forward/2) .\n\nFor example, you can have an authenticated router and a public router.\n\n```elixir\ndefmodule Public do\n  use Francis\n  get(\"/\", fn _ -> \"ok\" end)\nend\n\ndefmodule Private do\n  use Francis\n  import Plug.BasicAuth\n  plug(:basic_auth, username: \"test\", password: \"test\")\n  get(\"/\", fn _ -> \"hello\" end)\nend\n\ndefmodule TestApp do\n  use Francis\n\n  forward(\"/path1\", to: Public)\n  forward(\"/path2\", to: Private)\n\n  unmatched(fn _ -> \"not found\" end)\nend\n```\n\nCheck the folder [examples](https://github.com/francis-build/francis/tree/main/examples) to see examples of how to use Francis.","ref":"readme.html#example-of-multiple-routers"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.38.2"}}